/*
    $Id: weif.h 158 2010-09-19 08:38:25Z matwey $
    Copyright (C) 2010  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _WEIF_H
#define _WEIF_H

#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/vector_proxy.hpp>

#include <utility>

#include "spline.h"

namespace weif {

	using namespace std;
	using namespace boost::numeric::ublas;
	using boost::numeric::ublas::vector;

	class massdimm_weight {
	public:
		typedef double value_type;
		typedef vector< value_type > vector_type;
		typedef vector_type::size_type size_type;
		typedef vector_type result_type;

		typedef utils::ublas::cubic_spline< vector_type, vector< std::complex<double> >, vector_type, vector< std::complex<double> > > sfourier_type;
	public:
		massdimm_weight( const vector_type& mass_lambda, const vector_type& mass_response, const vector_type& dimm_lambda, const vector_type& dimm_response, const vector_type& ap_inn, const vector_type& ap_out, double dimmsize, double dimmbase );

		vector_type operator() ( const value_type& height ) const;
	private:
		vector_type ap_inn_;
		vector_type ap_out_;
		double dimmsize_;
		double dimmbase_;
		size_type mass_leff_index_;
		size_type dimm_leff_index_;
		double mass_leff_;
		double dimm_leff_;
		sfourier_type mass_sfourier_;
		sfourier_type dimm_sfourier_;
		double mass_freq_max_;
		double dimm_freq_max_;
	};

	/* make_weight( crv, geometry ) */
}

    struct linear_generator {
        typedef double result_type;
        typedef size_t size_type;

        linear_generator( result_type start, result_type step );
        result_type operator()( size_type i ) const;
        size_type size( result_type max ) const;
    private:
        result_type start_,step_;
    };

    struct lg_generator {
        typedef double result_type;
        typedef size_t size_type;

        lg_generator( result_type start, result_type lstep, result_type gstep );
        result_type operator()( size_type i ) const;
        size_type size( result_type max ) const;
    private:
        result_type start_,t_,q_;
        size_type max_;
    };

#endif // _WEIF_H
